#import "PCAccessoryWindow.h"
#import "PCView.h"
#import <QuartzCore/QuartzCore.h>

@class PCPopoverView;

@protocol PCPopoverDelegateProtocol;


@interface PCPopover : NSObject <PCAccessoryWindowDelegateProtocol, NSWindowDelegate>
{
#if TARGET_OS_IPHONE
	UIViewController *iContentViewController;
	UIView *iContentView;
#else
	NSViewController *iContentViewController;
	NSWindow *iWindow;
	NSView *iContentView;
#endif
	
	NSObject <PCPopoverDelegateProtocol>	*iDelegate;
   	NSRectEdge	iBubbleTriangleEdge; // -1 (not the preferred edge, but rather the inverse
	CGFloat 	iTriangleArrowOffset;
	CGFloat     iContentViewMargin;
    BOOL 		iNeedsLayout;
	BOOL        iClosing;
    
	NSView* iMainView;
	NSView* backgroundView;
	PCPopoverView *iBorderView;
    NSColor* iBackgroundColor;
    NSColor* iStrokeColor;
    BOOL iDrawHeader;
	BOOL isAnimatingBackground;
	Class iWindowClass;
	
	NSImage		*iCachedBGImage;
}

+ (CGFloat)headerHeight;

#if TARGET_OS_IPHONE
@property (nonatomic, retain) UIViewController *contentViewController;
@property (nonatomic, retain) UIView* contentView;
#else
@property (nonatomic, retain) NSViewController *contentViewController;
@property (nonatomic, retain) NSImage *cachedBGImage;
@property (nonatomic, retain) NSView* contentView;
@property (readonly) NSWindow *window; // don't use directly, consider private
#endif
@property (nonatomic, assign) CGSize contentSize;
@property (nonatomic, assign) CGFloat contentViewMargin;
@property (nonatomic, assign) NSObject <PCPopoverDelegateProtocol> *delegate;
@property (readonly) BOOL shown;
@property (readonly, getter = isClosing) BOOL closing;
@property (readonly) CGRect positioningRect; // current rect on screen
@property (nonatomic, retain) NSColor* backgroundColor;
@property (nonatomic, retain) NSColor* strokeColor;
@property BOOL drawHeader;
@property (nonatomic, assign) Class windowClass;

#if TARGET_OS_IPHONE
- (id)initWithContentViewController:(UIViewController*)contentViewController;
- (id)initWithContentView:(UIView*)contentView;

- (void)showRelativeToRect:(CGRect)rect ofView:(UIView*)view preferredEdge:(CGRectEdge)edge;
- (void)showRelativeToRect:(CGRect)rect ofView:(UIView*)view preferredEdge:(CGRectEdge)edge makeKey:(BOOL)makeKey;

#else
- (id)initWithContentViewController:(NSViewController*)contentViewController;
- (id)initWithContentView:(NSView*)contentView;

- (void)showRelativeToRect:(CGRect)rect ofView:(NSView*)view preferredEdge:(CGRectEdge)edge;
- (void)showRelativeToRect:(CGRect)rect ofView:(NSView*)view preferredEdge:(CGRectEdge)edge makeKey:(BOOL)makeKey;

#endif



- (void)close;
//- (void)performClose:(id)sender; // consults the delegate before closing

- (void)animateBackgroundColor:(CGColorRef)backgroundColor;

@end


@protocol PCPopoverDelegateProtocol
@optional

- (void)popoverDidClose:(PCPopover*)popover;
- (void)popoverDidShow:(PCPopover*)popover;
- (BOOL)popoverShouldClose:(PCPopover*)popover;
- (void)popoverWillClose:(PCPopover*)popover;
- (void)popoverWillShow:(PCPopover*)popover;

@end


@interface PCPopoverView : PCView 
{
	PCPopover *iPopover;
}

@property (nonatomic, assign) PCPopover *popover;

@end